use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
Alders
Alders'
App
AppVeyor
CISetup
ConfigFile
ConfigUpdater
Dave
Eilam
Eilam's
Elep
Fowler
Inc
MAXMIND
MAXMIND's
Mark
MaxMind
MaxMind's
Olaf
Oschwald
Oschwald's
PayPal
Role
Rolsky
Rolsky's
Travis
Types
Zak
appveyor
autarch
bin
dev
dir
distro
lib
mark
oalders
param
setup
travis
unthreaded
zakame
